package com.velocitypowered.api.event.player;

import com.google.common.base.Preconditions;
import com.velocitypowered.api.proxy.Player;
import com.velocitypowered.api.proxy.server.RegisteredServer;

/**
 * This event is fired once the player has successfully connected to the target server and the connection to the previous
 * server has been de-established.
 */
public final class ServerConnectedEvent {
    private final Player player;
    private final RegisteredServer server;

    public ServerConnectedEvent(Player player, RegisteredServer server) {
        this.player = Preconditions.checkNotNull(player, "player");
        this.server = Preconditions.checkNotNull(server, "server");
    }

    public Player getPlayer() {
        return player;
    }

    public RegisteredServer getServer() {
        return server;
    }

    @Override
    public String toString() {
        return "ServerConnectedEvent{" +
                "player=" + player +
                ", server=" + server +
                '}';
    }
}
